# Windows Powershell Script
# Will extract, configure and run a single thread on prime 95,
# using $process.ProcessorAffinity to assign to each cpu core for the specified time
# Assumes SMT is enabled, so one core means two threads and the process is assigned to two threads at a time
# Provided as-is and for use at your own risk.

$p95path="p95v303b6.win64.zip"; # path to p95 .zip you want to extract and use

$loops=3; # Number of times to loop arount all cores
$cycle_time=180; # Approx time in s to run on each core (180s matches included p95 config of 3 min test time)
$cooldown=15; # Time in s to cool down between testing each core
$stop_on_error=0; # If "0" continue to next core if error found.  If "1" stop.

# adjust next two values to limit testing to a specific range of cores
$first_core=0; # First core to test in each loop.  Any cores lower than this number will not be tested.
$last_core=31; # Last core to test in each loop.  MAX value=31.  Any cores (that exist) higher than this number will not be tested.
# NOTE: Cores 32 or higher will result in an Error: "Arithmetic operation resulted in an overflow."

$timestep=10; # Check for errors every this many seconds

# After extracting, we will add the following lines to local.txt for single thread, non-AVX test
#     NumCPUs=1
#     CpuNumHyperthreads=1
#     CpuSupportsAVX=0
#     CpuSupportsAVX2=0
#     CpuSupportsAVX512F=0
# Add the following lines to prime.txt for stress test with FFT size of 84
#     StressTester=1
#     UsePrimenet=0
#     MinTortureFFT=84
#     MaxTortureFFT=84
#     TortureMem=8
#     TortureTime=3

echo "Extracting prime95 from $p95path"

if (Test-Path $p95path)
{
    Expand-Archive -LiteralPath "$p95path" -DestinationPath p95 -ErrorAction SilentlyContinue
}
else
{
    echo "!!!! ============================================= !!!!"
    echo "!!!! $p95path not found"
    echo "!!!! Download and copy this into $pwd"
    echo "!!!! ============================================= !!!!"    
    Wait-Event    
    exit
}

echo "Configuring prime95 for single core, non-AVX torture test"
cp local.txt p95/
cp prime.txt p95/

$LogicalProcessors = Get-ComputerInfo -Property CsNumberOfLogicalProcessors
$num_cores = $LogicalProcessors.CsNumberOfLogicalProcessors/2

echo "Detected $num_cores cores"

if ($num_cores -gt $last_core)
{
    $num_cores = $last_core
}
echo "Testing cores $first_core through $num_cores"


if ((Get-Process -Name prime95 -ErrorAction SilentlyContinue).Count -gt 0)
{
    echo "!!!! ============================================= !!!!"
    echo "!!!! ERROR Prime95 is already running              !!!!"
    echo "!!!! ============================================= !!!!"    
} else
{
    echo "Moving any previous results into prev.results\"
    if (Test-Path .\core*failure.txt)
    {
        rmdir prev.results -Recurse -Force -ErrorAction SilentlyContinue
        mkdir prev.results -ErrorAction SilentlyContinue
        mv core*_failure.txt prev.results
    }
    if (Test-Path .\p95\results.txt)
    {
        del .\p95\prev.results.txt -ErrorAction SilentlyContinue
        mv .\p95\results.txt .\p95\prev.results.txt
    }

    echo "Looping $loops times around all cores"

    $first_run=1

    for ($i=1; $i -le $loops; $i++)
    {
        echo "Loop $i out of $loops"
        for ($core=$first_core; $core -lt $num_cores; $core++)
        {
            # skip testing if this core already failied in an earlier loop
            if (Test-Path "core${core}_loop*_failure.txt")
            {
                echo "!!!! ============================================= !!!!"
                echo "!!!! Skipping core ${core} due to previous failure !!!!"
                echo "!!!! ============================================= !!!!"
            }
            else
            {
                $timer=0
                $p95result=""
                
                # Don't cool down before the first test
                if ($first_run -eq 1)
                {
                    $first_run= 0
                }
                else
                {
                    echo "Cooling down for $cooldown seconds"
                    Start-Sleep -Seconds $cooldown
                }

                echo "Starting $cycle_time second torture test on core $core"
                Start-Process -FilePath .\p95\prime95.exe -ArgumentList "-T" -WindowStyle Minimized

                [Int64]$affinity=[Math]::Pow(2, $core*2) + [Math]::Pow(2, $core*2+1)

                $process=Get-Process prime95
                $process.ProcessorAffinity=[System.IntPtr]$affinity

        
                while ( (Get-Process -Name prime95 -ErrorAction SilentlyContinue).Count -eq 0 )
                {        
                    Start-Sleep -Milliseconds 100
                }

                # wait for p95 to run for $cycle_time, as long as there is no error
                while ( ($timer -lt $cycle_time) -and (-not($p95result)) -and (-not(Test-Path "core${core}_loop*_failure.txt")) )
                {
                    Start-Sleep -Seconds $timestep
                    $timer=$timer+$timestep
                    $p95result= if (Test-Path .\p95\results.txt) {Select-String .\p95\results.txt -Pattern ERROR}
                }
                if ($p95result)
                {
                    echo "!!!! ============================================= !!!!"
                    echo "!!!! Test FAILED on core $core.                    !!!!"
                    echo "!!!! Check core${core}_loop${loop}_failure.txt     !!!!"
                    echo "!!!! ============================================= !!!!"
                    echo "$p95result"
                    mv .\p95\results.txt "core${core}_loop${i}_failure.txt"
                    if ($stop_on_error -gt 0) 
                    { 
                        Stop-Process -Id $process.Id -ErrorAction SilentlyContinue
                        Wait-Event 
                    }
                } else
                {
                    echo "Test passed on core $core."            
                }
                Stop-Process -Id $process.Id
                Wait-Process -Id $process.Id -ErrorAction SilentlyContinue
            }
        }
    }
}

echo ""
echo "Testing complete."
echo "Check for any failures"
echo "ls core*_failure.txt"
echo (ls core*_failure.txt)

Wait-Event

